/*
    MakeMKV GUI - Graphics user interface application for MakeMKV

    Copyright (C) 2009-2010 GuinpinSoft inc <makemkvgui@makemkv.com>

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS"
    basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
    License for the specific language governing rights and limitations
    under the License.

*/
#import <Foundation/Foundation.h>
#import <NSUserNotification.h>

class QMainWindow;

#include <QtCore/QString.h>
#include "notify.h"

static NSUserNotificationCenter *NotificationCenter = nil;
static Class UserNotification = nil;
static unsigned int activity_time = 0;

void notifyInit()
{
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
    NSBundle* foundation = [NSBundle bundleWithIdentifier:@"com.apple.Foundation"];
    Class nclass;

    if (foundation != nil )
    {
        nclass = [foundation classNamed:@"NSUserNotificationCenter"];
        if (nclass != nil)
        {
            NotificationCenter = [nclass defaultUserNotificationCenter];
            if (NotificationCenter != nil)
            {
                UserNotification = [foundation classNamed:@"NSUserNotification"];
                if (UserNotification == nil)
                {
                    NotificationCenter = nil;
                }
            }
        }
    }
    [pool release];
}

void notifyStart(QMainWindow* mainWindow,unsigned long Id,const QString &Name)
{
    activity_time = 0;
}

void notifyUpdate(QMainWindow* mainWindow,unsigned int Value,unsigned int MaxValue,unsigned int TimeElapsed)
{
    if ((TimeElapsed-activity_time)>25000)
    {
        UpdateSystemActivity(0);
        activity_time = TimeElapsed;
    }
}

void notifyFinish(QMainWindow* mainWindow)
{
}

void notifyEvent(QMainWindow* mainWindow,unsigned long Id,const char* IdName,const QString &Text,const QString &Name)
{
    if (NotificationCenter != nil)
    {
        NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
        QByteArray  dataText = Text.toUtf8();
        NSString* stringText = [[NSString alloc] initWithUTF8String: dataText.data() ];
        QByteArray  dataName = Name.toUtf8();
        NSString* stringName = [[NSString alloc] initWithUTF8String: dataName.data() ];
        NSUserNotification* notification = [[UserNotification alloc] init];

        [notification setTitle: stringName];
        [notification setInformativeText: stringText];
        [notification setHasActionButton: NO];
        [notification setOtherButtonTitle: @""];

        [NotificationCenter deliverNotification: notification];

        [notification release];
        [stringName release];
        [stringText release];
        [pool release];
    }
}

void notifyCleanup()
{
}

